/*
 * @(#)DiagramFigure.java
 *
 */
package de.renew.diagram;

import CH.ifa.draw.figures.AttributeFigure;
import CH.ifa.draw.framework.Handle;
import CH.ifa.draw.standard.BoxHandleKit;
import CH.ifa.draw.util.StorableInput;
import CH.ifa.draw.util.StorableOutput;

import de.renew.gui.InscribableFigure;
import de.renew.shadow.ShadowNet;
import de.renew.shadow.ShadowNetElement;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.util.Vector;


/**
 * DiagramFigure is the underlying basic primitive of the entire Diagram figure drawing package.
 *  It provides default overriding implementations of {@link AttributeFigure} and lays the groundwork
 *  for extension into specific Figures with which AIP drawings are created and prepared for
 *  programmatic translation into Petri Networks.
 */
public class DiagramFigure extends AttributeFigure
                implements IDiagramElement, InscribableFigure {
    /*
     * Serialization support.
     */


    //    private static final long serialVersionUID = 184722075881789163L;

    /**
     * Determines position and size of the rectangle by
     * specifying position and size of its bounding box.
     * @serial
     **/
    public Rectangle fDisplayBox;

    /**
     * information field about serialization version
     */
    @SuppressWarnings("unused")
    private int rectangleFigureSerializedDataVersion = 1;

    /**
     * Create a new DiagramFigure at (x,y) Canvas (0,0) with extension (0,0) from that point.
     */
    public DiagramFigure() {
        this(new Point(0, 0), new Point(0, 0));
    }

    /**
     * Create a new DiagramFigure starting at the Origin and extending until the opposite corner.
     * @param origin the origin point where the Figure's display box starts
     * @param corner the other corner where the Figure's display box ends
     */
    public DiagramFigure(Point origin, Point corner) {
        basicDisplayBox(origin, corner);
    }

    @Override
    public void basicDisplayBox(Point origin, Point corner) {
        fDisplayBox = new Rectangle(origin);
        fDisplayBox.add(corner);
    }

    @Override
    public Vector<Handle> handles() {
        Vector<Handle> handles = new Vector<Handle>();
        BoxHandleKit.addHandles(this, handles);
        return handles;
    }

    @Override
    public Rectangle displayBox() {
        return new Rectangle(fDisplayBox.x, fDisplayBox.y, fDisplayBox.width,
                        fDisplayBox.height);
    }

    /**
     * Returns a default Dimension setting of 20 width bei 48 height
     * @return default Dimensioning for use with this DiagramFigure
     */
    public static Dimension defaultDimension() {
        return new Dimension(20, 48);
    }

    @Override
    protected void basicMoveBy(int x, int y) {
        fDisplayBox.translate(x, y);
    }

    @Override
    public void drawBackground(Graphics g) {
        Rectangle r = displayBox();
        Graphics2D g2 = (Graphics2D) g;
        Shape s = new Rectangle2D.Float(r.x, r.y, r.width, r.height);
        g2.fill(s);
    }

    @Override
    public void drawFrame(Graphics g) {
        Rectangle r = displayBox();
        Graphics2D g2 = (Graphics2D) g;
        Shape s = new Rectangle2D.Float(r.x, r.y, r.width, r.height);
        g2.draw(s);
    }

    //-- store / load ----------------------------------------------
    @Override
    public void write(StorableOutput dw) {
        super.write(dw);
        dw.writeInt(fDisplayBox.x);
        dw.writeInt(fDisplayBox.y);
        dw.writeInt(fDisplayBox.width);
        dw.writeInt(fDisplayBox.height);
    }

    @Override
    public void read(StorableInput dr) throws IOException {
        super.read(dr);
        fDisplayBox = new Rectangle(dr.readInt(), dr.readInt(), dr.readInt(),
                        dr.readInt());
    }

    @Override
    public ShadowNetElement buildShadow(ShadowNet net) {
        return null;
    }

    @Override
    public ShadowNetElement getShadow() {
        return null;
    }
}