/*
 * Created on Apr 16, 2003
 */
package de.renew.diagram;


import CH.ifa.draw.figures.TextFigure;
import CH.ifa.draw.framework.DrawingView;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.Handle;
import CH.ifa.draw.standard.LocatorHandle;
import CH.ifa.draw.standard.RelativeLocator;

import de.renew.gui.TokenHandle;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.Vector;

//TODO: declare abstract
/**
 * A DiagramTextFigure is a TextFigure that can be used in a DiagramDrawing.
 *
 * @author Lawrence Cabac
 */
public class DiagramTextFigure extends TextFigure implements IDiagramElement {
    /**
     * (default) hidden text of the figure, which is where a string is stored if the text is toggled to hidden
     */
    public static final String HIDDEN_TEXT = "HiddenText";
    /**
     * (Default) alternate text that is shown if the text is not toggled to hidden.
     */
    private static final String ALTERNATE_TEXT = "AlternateText";
    /**
     * Default isHidden string expression
     */
    public static final String IS_HIDDEN = "IsHidden";
    private static final String DUMMY = "~";

    //private boolean isClosed = false;
    // private String alternateText = "~";
    /**
     * Retrieve the alternative text string for this object
     * @return the alternateText set for this DiagramTextFigure
     */
    public String getAlternateText() {
        return (String) getAttribute(ALTERNATE_TEXT);
    }


    /**
     * Configure a new Alternative Text for this DiagramFigure to use.
     * @param alternateText the alternateText to set
     */
    public void setAlternateText(String alternateText) {
        setAttribute(ALTERNATE_TEXT, alternateText);
    }

    /**
     * Toggle whether the alternate text for this figure is visible or hidden. Every call of the method
     * toggles to the other state.
     */
    public void toggleText() {
        if (getAttribute(IS_HIDDEN) == null) {
            setAttribute(IS_HIDDEN, false);
        }
        setAttribute(IS_HIDDEN, !(Boolean) getAttribute(IS_HIDDEN));
        if ((Boolean) getAttribute(IS_HIDDEN)) {
            setAttribute(HIDDEN_TEXT, getText());
            setText((String) getAttribute(ALTERNATE_TEXT));
            setAttribute(IS_HIDDEN, true);
        } else {
            setText((String) getAttribute(HIDDEN_TEXT));
            setAttribute(IS_HIDDEN, false);
        }
    }

    /**
     * Set the figure text to hidden no matter the current state.
     */
    public void hideText() {
        if (getAttribute(IS_HIDDEN) == null) {
            setAttribute(IS_HIDDEN, false);
        }
        if (!(Boolean) getAttribute(IS_HIDDEN)) {
            setAttribute(HIDDEN_TEXT, getText());
            setText((String) getAttribute(ALTERNATE_TEXT));
            setAttribute(IS_HIDDEN, true);
        }
    }

    /**
     * Set the figure text to visible, no matter the current state.
     */
    public void unhideText() {
        if (getAttribute(IS_HIDDEN) == null) {
            setAttribute(IS_HIDDEN, false);
            return;
        }
        if ((Boolean) getAttribute(IS_HIDDEN)) {
            setText((String) getAttribute(HIDDEN_TEXT));
            setAttribute(IS_HIDDEN, false);
        }
    }

    /**
     * Return the text set for this figure to display if the text is toggled to not hidden
     * @return the String set for display when toggled to unhide
     */
    public String getRealText() {
        Boolean attribute = (Boolean) getAttribute(IS_HIDDEN);
        if (attribute != null && attribute) {
            return (String) getAttribute(HIDDEN_TEXT);
        } else {
            return super.getText();
        }
    }


    @Override
    public void drawFrame(Graphics g) {
        // TODO Auto-generated method stub
        super.drawFrame(g);
    }

    @Override
    public Vector<Handle> handles() {
        Vector<Handle> v = super.handles();

        //NOTICEsignature
        LocatorHandle handle = new DiagramTextHandle(this, Color.red,
                        Color.blue, new RelativeLocator(0.1, 0.0));
        //handle.draw(g);
        v.add(handle);
        return v;
    }

    /**
     * Default construction of a DiagramFigure, sets the Display-text to "DUMMY"
     */
    public DiagramTextFigure() {
        super();
        setAlternateText(DUMMY);
    }

    /**
     * Default construction of a DiagramFigure with the ability to be connected to other figures
     * according to the argument. Sets the Display text to "Dummy".
     * @param canBeConnected boolean control whether this DiagramTextFigure can be connected to or not. 
     */
    public DiagramTextFigure(boolean canBeConnected) {
        super(canBeConnected);
        setAlternateText(DUMMY);
    }

    /**
     * Configure the text figure text. Sets the alternate text for this DiagramTextFigure to "DUMMY".
     * @param text the text to configure in the supertype.
     */
    public DiagramTextFigure(String text) {
        super(text);
        setAlternateText(DUMMY);
    }

    /**
     * Configure the text figure text with explicit control over ReadOnly setting. Sets the alternate text for this DiagramFigure
     * to "DUMMY"
     * @param text the text to configure in the supertype
     * @param isReadOnly readOnly property control boolean
     */
    public DiagramTextFigure(String text, boolean isReadOnly) {
        super(text, isReadOnly);
        setAlternateText(DUMMY);
    }

    @Override
    public boolean inspect(DrawingView view, boolean alternate) {
        if (!alternate) {
            return super.inspect(view, false);
        } else {
            if (acceptsTyping()) {
                Object attr = getAttribute(IS_HIDDEN);
                if (attr != null && (Boolean) attr) {
                    setText((String) getAttribute(HIDDEN_TEXT));
                    setAttribute(IS_HIDDEN, false);
                }
                (view.editor()).doTextEdit(this);
                return true;
            }
            return false;
        }
    }

    private final class DiagramTextHandle extends LocatorHandle
                    implements TokenHandle {
        //NOTICEsignature
        private DiagramTextHandle(Figure owner, Color fillColor, Color penColor,
                                  CH.ifa.draw.framework.Locator l) {
            super(owner, l);
        }

        @Override
        public void invokeStart(MouseEvent e, int x, int y, DrawingView view) {
            //            System.out.println("Clicked");
            super.invokeStart(e, x, y, view);
            if (e.getClickCount() == 1) {
                toggleText();
            }
            // Despite the fact that the appearance of the FSFigure changes
            // when nodes are opened or closed, the semantics of the figure do
            // not change. More important, the FSFigure does currently not
            // serialize information about open nodes in its undo snapshot.
            // So we inform the AbstractTool that nothing happened.
            noChangesMade();
        }

        @Override
        public void draw(Graphics g) {
            // super.draw(g);
            Rectangle r = displayBox();
            g.setColor(Color.gray);
            g.fillRect(r.x, r.y, r.width, r.height);

            g.setColor(Color.red);
            g.drawRect(r.x, r.y, r.width, r.height);
        }
    }
}