package de.renew.diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.standard.AbstractConnector;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;


/**
 * A HBottomConnector is a special connector.
 * It locates connection points at the left or right bottom corner.
 *
 * {@link CH.ifa.draw.framework.Connector}
 * @author Lawrence Cabac
 */
public class HBottomConnector extends AbstractConnector {
    /*
     * Serialization support.
     */
    private static final long serialVersionUID = -1461450322712345462L;

    /**
     * Creates a new HBottomConnector instance.
     */
    public HBottomConnector() { // only used for Storable implementation
    }

    /**
     * Creates a HBottomConnector belonging to given owner
     *
     * @param owner figure to which this connector belongs
     */
    public HBottomConnector(Figure owner) {
        super(owner);
    }

    @Override
    public Point findStart(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.end().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(1), new Dimension(0, 0));
        }


        return chop(connection.start().owner(), r1);

    }

    @Override
    public Point findEnd(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.start().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(connection.pointCount() - 2),
                            new Dimension(0, 0));
        }
        return chop(connection.end().owner(), r1);
    }

    /**
     * Returns a Point at the bottom left ot bottom right of the target figure depending on the source.
     *
     * @param target the target figure
     * @param source the source rectangle
     * @return the Point at the bottom left or right
     */
    protected Point chop(Figure target, Rectangle source) {
        Rectangle r = target.displayBox();

        int horizontal = r.x;
        int vertical = r.y + r.height;


        if (source.x > r.x) {
            horizontal = r.x + r.width;
        }

        return new Point(horizontal, vertical);

    }

    /**
     * The display box of a <code>BottomConnector</code> is a
     * zero-height rectangle covering the bottom of the owner figure.
     *
     * @return the bottom rectangle of the owner figure.
     **/
    @Override
    public Rectangle displayBox() {
        Rectangle r = owner().displayBox();
        return new Rectangle(r.x, r.y + r.height, r.width, 0);
    }
}