package de.renew.diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.standard.AbstractConnector;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;


/**
 * HCenterConnector is a special connector entity used for horizontal center connections on
 * {@link DiagramFigure}. It provides special implementations of its start and ending locations towards
 * this end.
 *
 * {@link CH.ifa.draw.framework.Connector}
 * @author Lawrence Cabac
 */
public class HCenterConnector extends AbstractConnector {
    /**
     * Stub constructor provided for {@link CH.ifa.draw.util.Storable} implementation.
     */
    public HCenterConnector() { // only used for Storable implementation
    }

    /**
     * Create a new HCenterConnector.
     * @param owner the Figure this HCenterConnector is part of.
     */
    public HCenterConnector(Figure owner) {
        super(owner);
    }

    @Override
    public Point findStart(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.end().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(1), new Dimension(0, 0));
        }

        //NOTICEsignature
        return chop(connection.start().owner(), r1, connection.startPoint().y);

    }

    @Override
    public Point findEnd(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.start().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(connection.pointCount() - 2),
                            new Dimension(0, 0));
        }

        //NOTICEsignature
        return chop(connection.end().owner(), r1, connection.startPoint().y);
    }

    /**
     * Compute a point that "chops" the target figure entity's dimensions in half
     * on the vertical axis based on targeting of a source, constraining the horizontal
     * coordinates within the horizontal extension of the target figure.
     * @param target the target to chop on
     * @param source a source coordinate relative to which the connection will be computed
     * @param xxx will be ignored
     * @return a Point coordinate snapped to the target figure.
     */
    protected Point chop(Figure target, Rectangle source, int xxx) {
        Rectangle r = target.displayBox();

        int horizontal = r.x;
        int vertical = (int) (r.y + r.getHeight() / 2);


        if (source.x > r.x) {
            horizontal = r.x + r.width;
        }
        return new Point(horizontal, vertical);


    }

    /**
     * The display box of a <code>HCenterConnector</code> is a
     * zero-height rectangle covering the top of the owner figure.
     *
     * @return the top rectangle of the owner figure.
     **/
    @Override
    public Rectangle displayBox() {
        Rectangle r = owner().displayBox();
        return new Rectangle(r.x, (r.y + r.height / 2), r.width, 0);
    }
}