package de.renew.diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.standard.AbstractConnector;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;


/**
 * HSplitCenterConnector is a special connector class for horizontal figures which split message
 * connections. Towards this end it provides custom snapping implementations.
 *
 * {@link CH.ifa.draw.framework.Connector}
 * @author Lawrence Cabac
 * */
public class HSplitCenterConnector extends AbstractConnector {
    /**
     * Stub constructor for {@link CH.ifa.draw.util.Storable} implementation
     */
    public HSplitCenterConnector() { // only used for Storable implementation
    }

    /**
     * Create a new HSplitCenterConnector linked to a specific figure.
     * @param owner The Figure this Connector is part of.
     */
    public HSplitCenterConnector(Figure owner) {
        super(owner);
    }

    @Override
    public Point findStart(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.end().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(1), new Dimension(0, 0));
        }


        return chop(connection.start().owner(), r1);

    }

    @Override
    public Point findEnd(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.start().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(connection.pointCount() - 2),
                            new Dimension(0, 0));
        }
        return chop(connection.end().owner(), r1);
    }

    /**
     * Compute a "chop" on the target figure based on a source coordinate also given. In the case
     * of the HSplitCenterConnector, the connector will always return a coordinate at the top of bottom
     * of the figure, snapped to the center of the figure display box.
     * @param target the target to chop
     * @param source coordinates of connection request to consider for the chop
     * @return Chopped coordinate in canvas origin
     */
    protected Point chop(Figure target, Rectangle source) {
        Rectangle r = target.displayBox();
        int horizontal = (r.x + r.width / 2
                        - SplitDecoration.getDefaultSize() / 2);
        int vertical = (r.y + r.height / 2);

        if (source.x > r.x + r.width / 2) {
            horizontal += SplitDecoration.getDefaultSize();
        }
        return new Point(horizontal, vertical);


    }

    /**
     * The display box of a <code>TopConnector</code> is a
     * zero-height rectangle covering the top of the owner figure.
     *
     * @return the top rectangle of the owner figure.
     **/
    @Override
    public Rectangle displayBox() {
        Rectangle r = owner().displayBox();
        return new Rectangle(r.x, (r.y + r.height / 2), 0, 0);
    }
}