package de.renew.diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.standard.AbstractConnector;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;


/**
 * A HTopConnector locates the connection point at the top of a figure, either on its left or right side.
 *
 * {@link CH.ifa.draw.framework.Connector}
 * @author Lawrence Cabac
 */
public class HTopConnector extends AbstractConnector {
    /**
     * Serialization support.
     */
    private static final long serialVersionUID = -1461450322712345462L;

    /**
     * Stub constructor only used for {@link CH.ifa.draw.util.Storable} implementation
     */
    public HTopConnector() { // only used for Storable implementation
    }

    /**
     * Create a new HTopConnector on the owner figure.
     * @param owner The owner figure thus Connector is working for.
     */
    public HTopConnector(Figure owner) {
        super(owner);
    }

    @Override
    public Point findStart(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.end().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(1), new Dimension(0, 0));
        }


        return chop(connection.start().owner(), r1);

    }

    @Override
    public Point findEnd(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.start().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(connection.pointCount() - 2),
                            new Dimension(0, 0));
        }
        return chop(connection.end().owner(), r1);
    }

    /**
     * Compute a chop through the target figure, in this case so that the connector is located at the top left or the
     * top right of the target figure.
     * @param target the target figure to chop on
     * @param source a source coordinate to consider relative to the target figure during the chop.
     * @return Point coordinates in Canvas origin
     */
    protected Point chop(Figure target, Rectangle source) {
        Rectangle r = target.displayBox();

        int horizontal = r.x;
        int vertical = r.y;

        if (source.x > r.x) {
            horizontal = r.x + r.width;
        }

        return new Point(horizontal, vertical);


        //return Geom.angleToPoint(r, 
        //                       (r.intersection(source).equals(r) ? Math.PI : 0)
        //                     + (Geom.pointToAngle(r, Geom.center(source))));
    }

    /**
     * The display box of a <code>TopConnector</code> is a
     * zero-height rectangle covering the top of the owner figure.
     *
     * @return the top rectangle of the owner figure.
     **/
    @Override
    public Rectangle displayBox() {
        Rectangle r = owner().displayBox();
        return new Rectangle(r.x, r.y, r.width, 0);
    }
}