/*
 * Created on 14.05.2003
 */
package de.renew.diagram;

import CH.ifa.draw.framework.Connector;
import CH.ifa.draw.framework.Handle;

import java.util.Vector;


/**
 *<p>
 * SplitFigures underpin many layouts of the Diagram plugin. They are the generic ancestor of specific figure types
 * which may either split or join other figures together in more specific ways.
 * SplitFigures have a decorator providing generic decoration capabilities, can control and return various
 * connectors for the other figures they are related to, and provide Handles to be controlled with the GUI.
 *</p>
 * @author Lawrence Cabac
 */
public interface ISplitFigure {
    /**
     * Configure the FigureDecoration for the SplitFigure. This allows generic decorations
     * on this figure, with whatever FigureDecoration being configured being drawn when the SplitFigure is drawn.
     * Note {@link FigureDecoration} for constraints on the draw contract.
     * @param fd the FigureDecoration to use for decorating
     */
    void setDecoration(FigureDecoration fd);

    /**
     * Retrieves the FigureDecoration configured for this SplitFigure
     * @return the FigureDecoration object this object is configured with
     */
    FigureDecoration getDecoration();

    /**
     * Query whether the SplitFigure has a FigureConfiguration set or not.
     * @return False if no FigureDecoration is configured. True if one is configured.
     */
    boolean hasDecoration();

    /**
     * For the figure, return a new Connector object at the specified coordinates, relative to the top left corner
     * of the {@link CH.ifa.draw.framework.Figure displayBox} of the SplitFigure object.
     * @param x the X coordinate in the local frame with origin at the displayBox top left
     * @param y the Y coordinate in the local frame with origin at the displayBox top left
     * @return the Connector for this SplitFigure object at the specified coordinates
     */
    Connector connectorAt(int x, int y);

    /**
     * Return the Handles that allow the direct manipulation of this SplitFigure.
     * @return the Vector of {@link Handle} objects for the Figure 
     */
    Vector<Handle> handles();

    /**
     * Configure a fill color for the SplitFigure object.
     * @param col the awt.Color object representing the color that fills this object.
     */
    void setFillColor(java.awt.Color col);
}