package de.renew.diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.standard.AbstractConnector;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;


/**
 * A LeftConnector is a special connector class.
 * It locates connection points on the left side of a figure.
 *
 * {@link CH.ifa.draw.framework.Connector}
 * @author Lawrence Cabac
 */
public class LeftConnector extends AbstractConnector {
    /**
     * Stub constructor only used for {@link CH.ifa.draw.util.Storable} implementation
     */
    public LeftConnector() { // only used for Storable implementation
    }

    /**
     * Create a new LeftConnector on a figure.
     * @param owner the figure the LeftConnector is part of.
     */
    public LeftConnector(Figure owner) {
        super(owner);
    }

    @Override
    public Point findStart(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.end().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(1), new Dimension(0, 0));
        }


        return chop(connection.start().owner(), r1);

    }

    @Override
    public Point findEnd(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.start().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(connection.pointCount() - 2),
                            new Dimension(0, 0));
        }
        return chop(connection.end().owner(), r1);
    }

    /**
     * Calculate a chop through the target figure under consideration of the relative position
     * of the source entity. The computed coordinate will be at the horizontal left edge of the
     * target figure and depending on the position of source, either at the top or bottom.
     * @param target the target object on which to chop
     * @param source the source whose relative position to the target to consider in the chop.
     * @return the chopped coordinates
     */
    protected Point chop(Figure target, Rectangle source) {
        Rectangle r = target.displayBox();

        int horizontal = r.x;
        int vertical = r.y + r.height;
        if (source.y < r.y + r.height / 2) {
            vertical = r.y;
        }
        return new Point(horizontal, vertical);


    }

    /**
     * The display box of a <code>LeftConnector</code> is a
     * zero-height rectangle covering the left of the owner figure.
     *
     * @return the left rectangle of the owner figure.
     **/
    @Override
    public Rectangle displayBox() {
        Rectangle r = owner().displayBox();
        return new Rectangle(r.x, r.y + r.height, 0, 0);
    }
}