/*
 * Created on May 14, 2003
 */
package de.renew.diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Connector;
import CH.ifa.draw.framework.DrawingEditor;
import CH.ifa.draw.framework.Figure;

import java.awt.event.MouseEvent;


/**
 * Draws a new Message Connection between two TaskFigures or HSplitFigures
 *
 * @author Lawrence Cabac
 */
public class MessageConnectionTool extends ConnectionTool {
    /**
     * Static logger entity for all instances of this class.
     */
    public static org.apache.log4j.Logger logger = org.apache.log4j.Logger
                    .getLogger(MessageConnectionTool.class);

    /**
     * Create a new MessageConnectionTool within the given context and with the given prototype to clone.
     * @param editor the editor context within which to work
     * @param prototype the prototype to clone
     */
    public MessageConnectionTool(DrawingEditor editor,
                                 ConnectionFigure prototype) {
        super(editor, prototype);
    }

    @Override
    protected Connector findConnector(int x, int y, Figure f) {
        if (f instanceof TaskFigure) {
            return ((TaskFigure) f).messageConnectorAt();
        }
        else {
            return f.connectorAt(x,y);
        }
    }

    /**
     * Connects the figures if the mouse is released over another
     * figure.
     * @param e mouse event, will be queried for its current coordinates
     * @param x the x coordinates, will be ignored
     * @param y the y coordinates, will be ignored
     */
    @Override
    public void mouseUp(MouseEvent e, int x, int y) {
        Figure c = null;
        if (getFStartConnector() != null) {
            c = findTarget(e.getX(), e.getY(), drawing());
        }

        if (c != null) {
            setFEndConnector(findConnector(e.getX(), e.getY(), c));
            if (getFEndConnector() != null) {
                getFConnection().connectStart(getFStartConnector());
                getFConnection().connectEnd(getFEndConnector());
                getFConnection().updateConnection();

                MessageConnection message = (MessageConnection) getFConnection();
                DiagramFigure start = (DiagramFigure) message.startFigure();
                if (start instanceof TaskFigure) {
                    TaskFigure task = (TaskFigure) start;
                    logger.debug("start (Place) " + task);
                    task.addPeerName("out");
                }
                DiagramFigure end = (DiagramFigure) message.endFigure();
                if (end instanceof TaskFigure) {
                    TaskFigure task = (TaskFigure) end;
                    task.addPeerName("in");
                }
            }
            if (generatePeerInstantaneously(view())) {
                ((TailFigure) c).generatePeers();
            }
        } else if (getFConnection() != null) {
            view().remove(getFConnection());
            noChangesMade();
        }


        setFConnection(null);
        setFStartConnector(null);
        setFEndConnector(null);
        editor().toolDone();
    }
}