package de.renew.diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.standard.AbstractConnector;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;


/**
 * A RightBottomConnector is a special connector class.
 * It locates connection points at the right bottom corner.
 *
 * {@link CH.ifa.draw.framework.Connector}
 * @author Lawrence Cabac
 */
public class RightBottomConnector extends AbstractConnector {

    /**
     * Creates a new default RightBottomConnector.
     */
    public RightBottomConnector() { // only used for Storable implementation
    }

    /**
     * Creates a new RightBottomConnector with an owner.
     *
     * @param owner The owner of the connector
     */
    public RightBottomConnector(Figure owner) {
        super(owner);
    }

    @Override
    public Point findStart(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.end().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(1), new Dimension(0, 0));
        }

        //NOTICEsignature
        return chop(connection.start().owner(), r1);
    }

    @Override
    public Point findEnd(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.start().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(connection.pointCount() - 2),
                            new Dimension(0, 0));
        }

        //NOTICEsignature
        return chop(connection.end().owner(), r1);
    }

    /**
     * Compute a coordinate at the right bottom corner of the target figure.
     *
     * @param target the target figure whose right bottom point is to be returned
     * @param source will be ignored
     * @return The right bottom coordinate of the submitted target figure
     */
    //NOTICEsignature
    protected Point chop(Figure target, Rectangle source) {
        Rectangle r = target.displayBox();


        int horizontal = r.x + r.width;
        int vertical = r.y + r.height;

        return new Point(horizontal, vertical);
    }

    /**
     * The display box of a <code>RightBottomConnector</code> is a
     * zero-height, zero-width rectangle positioned at the right bottom
     * corner of the owner figure.
     *
     * @return the right bottom rectangle of the owner figure.
     **/
    @Override
    public Rectangle displayBox() {
        Rectangle r = owner().displayBox();
        return new Rectangle(r.x + r.width, r.y + r.height, 0, 0);
    }
}