package de.renew.diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.standard.AbstractConnector;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;


/**
 * A RightConnector is a special connector class.
 * It locates connection points on the right side of a figure.
 *
 * {@link CH.ifa.draw.framework.Connector}
 * @author Lawrence Cabac
 */
public class RightConnector extends AbstractConnector {

    /**
     * Creates a new default RightConnector.
     */
    public RightConnector() { // only used for Storable implementation
    }

    /**
     * Creates a new RightConnector with an owner.
     *
     * @param owner The owner of the connector
     */
    public RightConnector(Figure owner) {
        super(owner);
    }

    @Override
    public Point findStart(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.end().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(1), new Dimension(0, 0));
        }


        return chop(connection.start().owner(), r1);

    }

    @Override
    public Point findEnd(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.start().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(connection.pointCount() - 2),
                            new Dimension(0, 0));
        }
        return chop(connection.end().owner(), r1);
    }

    /**
     * Computes a coordinate on the right side of the target figure
     * under consideration of the relative position of the source entity.
     * The computed coordinate will be at the horizontal right edge of the
     * target figure and depending on the position of source, either at the top or bottom.
     *
     * @param target the target figure whose right point is to be returned
     * @param source the source whose relative position to the target to consider in the chop.
     * @return The coordinate on the right side of the submitted target figure
     */
    protected Point chop(Figure target, Rectangle source) {
        Rectangle r = target.displayBox();

        int horizontal = r.x + r.width;
        int vertical = r.y + r.height;
        if (source.y < r.y + r.height / 2) {
            vertical = r.y;
        }
        return new Point(horizontal, vertical);
    }

    /**
     * The display box of a <code>RightConnector</code> is a
     * zero-height rectangle covering the right of the owner figure.
     *
     * @return the right rectangle of the owner figure.
     **/
    @Override
    public Rectangle displayBox() {
        Rectangle r = owner().displayBox();
        return new Rectangle(r.x + r.width, r.y + r.height, 0, 0);
    }
}