package de.renew.diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.standard.AbstractConnector;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;


/**
 * VSplitCenterConnector is a special semantic extension of the {@link AbstractConnector} for use by {@link VSplitFigure}
 * to extend centered {@link LifeLineConnection} with other {@link TailFigure} objects.
 *
 * {@link CH.ifa.draw.framework.Connector}
 * @author Lawrence Cabac
 */
public class VSplitCenterConnector extends AbstractConnector {
    /**
     * Stub constructor for Storable implementation
     */
    public VSplitCenterConnector() { // only used for Storable implementation
    }

    /**
     * Create a new VSplitCenterConnector for the given owner figure
     * @param owner the owner of this connector
     */
    public VSplitCenterConnector(Figure owner) {
        super(owner);
    }

    @Override
    public Point findStart(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.end().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(1), new Dimension(0, 0));
        }


        return chop(connection.start().owner(), r1);

    }

    @Override
    public Point findEnd(ConnectionFigure connection) {
        Rectangle r1;
        if (connection.pointCount() == 2) {
            r1 = connection.start().displayBox();
        } else {
            r1 = new Rectangle(connection.pointAt(connection.pointCount() - 2),
                            new Dimension(0, 0));
        }
        return chop(connection.end().owner(), r1);
    }

    /**
     * Chop the target figure to locate the position of the connector on the figure according to the
     * contract opf the specific connector. In this case, a VSplitCenterConnector will locate itself
     * either on the top or bottom of a figure, in the horizontal center.
     * @param target the target figure to chop on
     * @param source location relative to the target figure to consider for the chop
     * @return the computed location for the Connector
     */
    protected Point chop(Figure target, Rectangle source) {
        Rectangle r = target.displayBox();
        int horizontal = r.x + (r.width / 2);
        int vertical = (r.y + r.height / 2
                        - SplitDecoration.getDefaultSize() / 2);
        if (source.y > r.y + r.height / 2) {
            vertical += SplitDecoration.getDefaultSize();
        }
        return new Point(horizontal, vertical);


    }

    /**
     * The display box of a <code>TopConnector</code> is a
     * zero-height rectangle covering the top of the owner figure.
     *
     * @return the top rectangle of the owner figure.
     **/
    @Override
    public Rectangle displayBox() {
        Rectangle r = owner().displayBox();
        return new Rectangle(r.x, r.y, r.width, 0);
    }
}