/*
 *
 *
 */
package de.renew.diagram;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.GeneralPath;


/**
 * XORDecoration is a combined appearance and AIP semantics specialization of {@link SplitDecoration} for the purpose
 * of configuring the appearance and generative behavior of {@link SplitFigure} Diagram figures. It implements the
 * behavior of an XOR split/join.
 */
public class XORDecoration extends SplitDecoration {
    /**
     * Create a new XOR decoration of standard size and coloration settings.
     */
    public XORDecoration() {
        setSize(20);
        setFillColor(Color.white);
        setFrameColor(Color.black);
    }

    /**
     * Draws the Decoration. The two points describe a rectangle, which is
     * the bounding box of the diamond.
     * @param g the graphics context to use
     * @param x the x coordinate to work at
     * @param y the y coordinate to work at
     * @param fillColor the filler color
     * @param lineColor the line color
     */
    @Override
    public void draw(Graphics g, int x, int y, Color fillColor,
                     Color lineColor) {
        super.draw(g, x, y, fillColor, lineColor);
        int size = getSize();
        int halfSize = size / 2;

        ((Graphics2D) g).draw(xline(x - halfSize, y - halfSize, size, size));
    }

    /**
     * Produces a new xline shape within the coordinate points
     * @param x1 upper left corner x coordinate
     * @param y1 upper left corner y coordinate
     * @param x2 lower right corner x coordinate
     * @param y2 lower right corner y coordinate
     * @return a new xline object.
     */
    public Shape xline(int x1, int y1, int x2, int y2) {
        return xline(new Rectangle(x1, y1, x2, y2));
    }

    /**
     * An x-line is a crossed X line produced within the confines of a rectangle.
     * @param r the rectangle within which to produce the x-line
     * @return the x-line object, drawn as a Shape.
     */
    public Shape xline(Rectangle r) {
        GeneralPath p = new GeneralPath();
        int h4 = r.height / 4;
        int h34 = r.height * 3 / 4;
        int w4 = r.width / 4;
        int w34 = r.width * 3 / 4;
        int a = r.x + w4;
        int b = r.x + w34;
        int u = r.y + h4;
        int v = r.y + h34;


        p.moveTo(a, v);
        p.lineTo(b, u);
        p.lineTo(r.x + r.width / 2, r.y + r.height / 2);
        p.lineTo(a, u);
        p.lineTo(b, v);
        p.lineTo(r.x + r.width / 2, r.y + r.height / 2);
        p.closePath();
        return p;
    }
}