package de.renew.diagram.commands;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.DrawApplication;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.util.Command;

import de.renew.diagram.ActionTextFigure;
import de.renew.diagram.DCServiceTextFigure;
import de.renew.diagram.DiagramTextFigure;


/**
 * This class provides a command to hide the own text of the figures and
 * show the alternative text of DiagramTextFigures and DCServiceTextFigure instead.
 *
 * @author Lawrence Cabac
 */
public class HideTextCommand extends Command {

    /**
     * Creates a new HideTextCommand with the given name.
     *
     * @param name The name of the command
     */
    public HideTextCommand(String name) {
        super(name);
    }

    @Override
    public void execute() {
        DrawApplication app = DrawPlugin.getGui();
        if (app != null) {
            Drawing drawing = app.drawing();
            FigureEnumeration fe = drawing.figures();
            while (fe.hasMoreElements()) {
                Figure fig = fe.nextFigure();
                if (fig instanceof ActionTextFigure
                                || fig instanceof DCServiceTextFigure) {
                    DiagramTextFigure textf = (DiagramTextFigure) fig;
                    textf.hideText();
                }
            }
            drawing.checkDamage();
        }
    }

    @Override
    public boolean isExecutable() {
        DrawApplication app = DrawPlugin.getGui();
        if (app != null) {
            if (app.drawing() == null) {
                return false;
            }
        }
        return super.isExecutable();
    }
}