/*
 * Created on Apr 26, 2003
 */
package de.renew.diagram.peer;

import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.io.DrawingFileHelper;
import CH.ifa.draw.io.StatusDisplayer;

import de.renew.io.RNWFileFilter;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;


/**
 * This class allows to load a net component during runtime from the tools-directory.
 *
 * @author Lawrence Cabac
 */
public class NCLoader {
    /**
     * Logger entity for all instances of this class.
     */
    public static org.apache.log4j.Logger logger = org.apache.log4j.Logger
                    .getLogger(NCLoader.class);
    static private Map<String, NCLoader> loaderMap = new HashMap<>();
    static private final StatusDisplayer sd = new StatusDisplayer() {
        @Override
        public void showStatus(String string) {
            logger.info(string);
        }
    };
    private String path;

    private NCLoader(String path) {
        super();
        this.path = path;
        logger.debug("Path " + path);
    }

    /**
     * Gets the current NCLoader.
     *
     * @param pluginLocation The location the jar-File of this plugin is in.
     * @return the current NCLoader.
     */
    static public NCLoader getInstance(URL pluginLocation) {
        return getInstance(pluginLocation, "tools");
    }

    /**
     * Gets the current NCLoader. Creates one if ist doesn't exist.
     *
     * @param pluginLocation The location the jar-File of this plugin is in.
     * @param toolsDirectory The location to the net components as rnw-files.
     * @return the current NCLoader. Creates one if ist doesn't exist.
     */
    static public NCLoader getInstance(URL pluginLocation,
                                       String toolsDirectory) {
        String path = new File(pluginLocation.getPath()).getParent() + "/"
                        + toolsDirectory + "/";

        NCLoader loader;
        if (loaderMap.containsKey(path)) {
            loader = loaderMap.get(path);
        } else {
            loader = new NCLoader(path);
            loaderMap.put(path, loader);
        }
        return loader;
    }

    /**
     * Gets the figures of a drawing located at the given path as String.
     *
     * @param drawingLocation The path to the drawing as String.
     * @return the figures of the drawing
     */
    public FigureEnumeration getFigures(String drawingLocation) {
        Drawing drawing = loadDrawing(drawingLocation);
        logger.debug("Drawing : " + drawing);
        if (drawing == null) {
            return null;
        }
                        //figures are single elements of a drawing
        return drawing.figures();
    }

    /*
     * Loads a drawing from the given location.
     * @param drawingLocation The path to the drawing as String.
     */
    private Drawing loadDrawing(String drawingLocation) {
        File file = DrawingFileHelper.checkAndAddExtension(
                        new File(path, drawingLocation), new RNWFileFilter());
        URL url = null;
        try {
            url = file.toURI().toURL();
        } catch (MalformedURLException e1) {
            if (logger.isDebugEnabled()) {
                logger.debug(NCLoader.class.getSimpleName() + ": ", e1);
            }
        }
        if (logger.isDebugEnabled()) {
            logger.debug(NCLoader.class.getSimpleName() + " Loading component: "
                            + url.toString());
        }
        return DrawingFileHelper.loadDrawing(url, sd);
    }
}