package diagram;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Connector;
import CH.ifa.draw.framework.DrawingEditor;
import CH.ifa.draw.framework.Figure;
import de.renew.diagram.HBottomConnector;
import de.renew.diagram.HSplitCenterConnector;
import de.renew.diagram.HSplitFigure;
import de.renew.diagram.HTopConnector;
import de.renew.diagram.HorizontalConnector;
import de.renew.diagram.MessageConnection;
import de.renew.diagram.MessageConnectionTool;
import de.renew.diagram.TaskFigure;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;

import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.verify;

/**
 * This class is a test class for {@link MessageConnectionTool}.
 * It tests if the right connector is found.
 *
 * @author Lukas Seifert
 */
public class MessageConnectionToolTest {

    /**
     * This inner class provides locally scoped access to protected methods of {@link MessageConnectionTool}.
     */
    static class TestMessageConnectionTool extends MessageConnectionTool {

        /**
         * Create a new TestMessageConnectionTool within the given context and with the given prototype to clone.
         *
         * @param editor the editor context within which to work
         * @param prototype the prototype to clone
         */
        public TestMessageConnectionTool(DrawingEditor editor, ConnectionFigure prototype) {
            super(editor, prototype);
        }

        @Override
        protected Connector findConnector(int x, int y, Figure f) {
            return super.findConnector(x, y, f);
        }
    }

    @Mock
    private DrawingEditor mockEditor;
    @Mock
    private MessageConnection testConnection;
    private TestMessageConnectionTool testMessageConnectionTool;

    @BeforeEach
    public void setUp() {
        testConnection = mock(MessageConnection.class);
        mockEditor = mock(DrawingEditor.class);

        testMessageConnectionTool = spy(new TestMessageConnectionTool(mockEditor, testConnection));
    }

    /**
     * This method tests the method findConnector from {@link MessageConnectionTool}.
     * It ensures that if the given figure is a TaskFigure the method {@link TaskFigure#messageConnectorAt()}
     * is called and a HorizontalConnector is returned.
     * If the given figure is a HSplitFigure the method {@link HSplitFigure#connectorAt(int, int)}
     * shall be called and a HBottomConnector, HTopConnector or HSplitCenterConnector returned, belonging on the coordinates.
     * In the upper part on the default box created at (0,0) this is a HTopConnector,
     * in the lower part a HBottomConnector and in the middle a HSplitCenterConnector.
     */
    @Test
    public void findConnectorTest() {
        TaskFigure testTaskFigure = spy(TaskFigure.class);
        HSplitFigure testHSplitFigure = spy(HSplitFigure.class);

        Connector connector = testMessageConnectionTool.findConnector(5, 5, testHSplitFigure);
        assertTrue(connector instanceof HBottomConnector);
        verify(testHSplitFigure).connectorAt(5, 5);

        connector = testMessageConnectionTool.findConnector(-5, -5, testHSplitFigure);
        assertTrue(connector instanceof HTopConnector);
        verify(testHSplitFigure).connectorAt(-5, -5);

        connector = testMessageConnectionTool.findConnector(0, 0, testHSplitFigure);
        assertTrue(connector instanceof HSplitCenterConnector);
        verify(testHSplitFigure).connectorAt(0, 0);

        connector = testMessageConnectionTool.findConnector(5, 5, testTaskFigure);
        assertTrue(connector instanceof HorizontalConnector);
        verify(testTaskFigure).messageConnectorAt();
    }

}