package diagram;

import CH.ifa.draw.framework.Connector;
import de.renew.diagram.BottomConnector;
import de.renew.diagram.HorizontalConnector;
import de.renew.diagram.TaskFigure;
import de.renew.diagram.TopConnector;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

/**
 * This class is a test class for {@link TaskFigure}.
 * It tests if connections are added at the right connectors.
 *
 * @author Lukas Seifert
 */
public class TaskFigureTest {

    private TaskFigure testTaskFigure;

    @BeforeEach
    public void setUp() {
        testTaskFigure = new TaskFigure();
    }

    /**
     * This method tests if the method {@link TaskFigure#connectorAt(int, int)}
     * returns a Top- or BottomConnector and not a HorizontalConnector.
     * It also tests if it returns a TopConnector when the coordinate is in the upper half
     * and a BottomConnector it the coordinate is in the lower half.
     * The half from the default box is at the coordinate (0, 0).
     */
    @Test
    public void connectorAtTest() {
        //make sure it gives back a top or bottom connector
        Connector connector = testTaskFigure.connectorAt(100, 0);
        assertTrue(connector instanceof BottomConnector);
        assertEquals(connector.owner(), testTaskFigure);

        connector = testTaskFigure.connectorAt(100, 1);
        assertTrue(connector instanceof BottomConnector);
        assertEquals(connector.owner(), testTaskFigure);

        connector = testTaskFigure.connectorAt(100, -1);
        assertTrue(connector instanceof TopConnector);
        assertEquals(connector.owner(), testTaskFigure);

        connector = testTaskFigure.connectorAt(100, 100);
        assertTrue(connector instanceof BottomConnector);
        assertEquals(connector.owner(), testTaskFigure);

        connector = testTaskFigure.connectorAt(100, -100);
        assertTrue(connector instanceof TopConnector);
        assertEquals(connector.owner(), testTaskFigure);
    }

    /**
     * This method tests if the method {@link TaskFigure#messageConnectorAt()}
     * returns a HorizontalConnector with the test instance as owner.
     * It also ensures the connector is not a Top- or BottomConnector
     */
    @Test
    public void messageConnectorAtTest() {
        Connector connector = testTaskFigure.messageConnectorAt();
        assertTrue(connector instanceof HorizontalConnector);
        assertEquals(connector.owner(), testTaskFigure);
    }

}