/*
 * Created on 18.08.2004
 */
package de.renew.logging.gui;

import bibliothek.gui.dock.common.DefaultSingleCDockable;
import bibliothek.gui.dock.common.SingleCDockable;
import bibliothek.gui.dock.common.event.CVetoClosingEvent;
import bibliothek.gui.dock.common.event.CVetoClosingListener;

import de.renew.gui.ComponentRenderer;
import de.renew.gui.JComponentCellEditor;
import de.renew.windowmanagement.Workbench;
import de.renew.windowmanagement.WorkbenchImpl;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.ComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.WindowConstants;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;


/**
* GUI for showing the trace messages produced during the
* simulation of petri nets
*
* @author Sven Offermann
*/
public class LoggingFrame {
    /**
     * {@link org.apache.log4j.Logger}
     */
    public static org.apache.log4j.Logger logger = org.apache.log4j.Logger
                    .getLogger(LoggingFrame.class); // TODO: antipattrn, no-one should be allowed to make logging-statements in the name of this class.
    /**
     * {@link LoggingController}
     */
    private LoggingController controller;
    /**
     * {@link JTable}
     */

    private JTable loggerTable;
    /**
     * {@link JScrollPane}
     */

    private JScrollPane scrollPane;
    /**
     * {@link JComboBox}
     */

    private JComboBox loggerComboBox;
    /**
     * {@link JCheckBox}
     */

    private JCheckBox updateCheckBox;
    /**
     * {@link JPanel}
     */

    private JPanel panel;

    /**
     * Creates a LoggingFrame, initializes it and makes it closable.
     * @param controller {@link LoggingController}
     * @param loggerNames Array of loggers.
     * @param workbench {@link Workbench}
     */
    public LoggingFrame(LoggingController controller, String[] loggerNames,
                        Workbench workbench) {
        this.controller = controller;
        panel = new JPanel();
        initGUI(loggerNames);
        DefaultSingleCDockable dockable = new DefaultSingleCDockable(
                        "Simulation Trace", panel);
        dockable.setCloseable(true);
        dockable.addVetoClosingListener(new CVetoClosingListener() {
            @Override
            public void closing(CVetoClosingEvent arg0) {
            }

            @Override
            public void closed(CVetoClosingEvent arg0) {
                SingleCDockable dockable = (SingleCDockable) arg0
                                .getDockable(0);
                dockable.getControl().removeDockable(dockable);

            }
        });
        workbench.addViewWindow(dockable, WorkbenchImpl.UPPER_LEFT_GROUP);
    }

    /**
     * initialize the Logger-Gui
     * @param loggerNames array with the loggers.
     */
    public void initGUI(String[] loggerNames) {
        try {
            loggerComboBox = new JComboBox(loggerNames);
            loggerComboBox.addActionListener(new ActionListener() { //TODO: use lambda-expression
                @Override
                public void actionPerformed(ActionEvent e) {
                    JComboBox cb = (JComboBox) e.getSource();
                    String loggerName = (String) cb.getSelectedItem();
                    controller.changeLogger(loggerName);
                }
            });

            scrollPane = new JScrollPane();

            loggerTable = new JTable(new TableModel(true)) {
                @Override
                public TableCellRenderer getCellRenderer(int row, int column) {
                    TableColumn tableColumn = getColumnModel()
                                    .getColumn(column);
                    TableCellRenderer renderer = tableColumn.getCellRenderer();

                    if (renderer == null) {
                        Class<?> c = getColumnClass(column);
                        if (c.equals(Object.class)) {
                            Object o = getValueAt(row, column);
                            if (o != null) {
                                c = getValueAt(row, column).getClass();
                            }
                        }
                        renderer = getDefaultRenderer(c);
                    }

                    return renderer;
                }

                @Override
                public TableCellEditor getCellEditor(int row, int column) {
                    TableColumn tableColumn = getColumnModel()
                                    .getColumn(column);
                    TableCellEditor editor = tableColumn.getCellEditor();

                    if (editor == null) {
                        Class<?> c = getColumnClass(column);
                        if (c.equals(Object.class)) {
                            Object o = getValueAt(row, column);
                            if (o != null) {
                                c = getValueAt(row, column).getClass();
                            }
                        }
                        editor = getDefaultEditor(c);
                    }

                    return editor;
                }
            };

            TableCellRenderer defaultRenderer = loggerTable
                            .getDefaultRenderer(JComponent.class);
            loggerTable.setDefaultRenderer(JComponent.class,
                            new ComponentRenderer(defaultRenderer));
            TableCellEditor defaultEditor = loggerTable
                            .getDefaultEditor(JComponent.class);
            loggerTable.setDefaultEditor(JComponent.class,
                            new JComponentCellEditor(defaultEditor, null));

            //            setName("Simulation Trace");
            BorderLayout thisLayout = new BorderLayout();
            panel.setLayout(thisLayout);
            // this.setSize(new java.awt.Dimension(345,262));
            panel.add(loggerComboBox, BorderLayout.NORTH);

            panel.add(scrollPane, BorderLayout.CENTER);

            scrollPane.add(loggerTable);
            scrollPane.setViewportView(loggerTable);

            updateCheckBox = new JCheckBox("permanent update");
            updateCheckBox.setSelected(true);
            updateCheckBox.addActionListener(new AbstractAction() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    controller.setPermanentUpdate(updateCheckBox.isSelected());
                }
            });
            panel.add(updateCheckBox, BorderLayout.SOUTH);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
    }

    /**
     *
     * @return {@link ComboBoxModel}
     */
    public ComboBoxModel getComboBoxModel() {
        return this.loggerComboBox.getModel();
    }

    /**
     *
     * @param dm {@link LoggerTableModel}
     */
    public void setTableModel(LoggerTableModel dm) {
        if (dm != null) {
            this.loggerTable.setModel(dm);
            dm.addTableModelListener(new TableModelListenerImpl());
            updateRowHeights();
        }
    }

    /**
     *
     * @return {@link LoggerTableModel}
     */
    public LoggerTableModel getTableModel() {
        return (LoggerTableModel) this.loggerTable.getModel();
    }

    /**
     * @return the selected Item of the combobox.
     */
    public String getSelectedLoggerName() {
        return (String) loggerComboBox.getSelectedItem();
    }

    /**
     * Updates the heights of the rows.
     * Iterates over the {@link TableModel} found in the {@link JTable} of this object.
     * Uses the preferred Size given by the {@link JComponent} found in the Table.
     */
    public void updateRowHeights() {
        TableModel tm = (TableModel) this.loggerTable.getModel();
        for (int x = 0; x < tm.getRowCount(); x++) {
            Object o = tm.getValueAt(x, 0);
            if (o instanceof JComponent) {
                JComponent c = (JComponent) o;
                this.loggerTable.setRowHeight(x, c.getPreferredSize().height);
            }
        }
    }

    /**
     * Set the used row height for the Frame.
     * @param row integer
     */
    public void updateRowHeight(int row) {
        TableModel tm = (TableModel) this.loggerTable.getModel();
        Object o = tm.getValueAt(row, 0);
        if (o instanceof JComponent) {
            JComponent c = (JComponent) o;
            this.loggerTable.setRowHeight(row, c.getPreferredSize().height);
        }
    }

    /**
     * implementation of the TableModelListener
     */
    private class TableModelListenerImpl implements TableModelListener {
        @Override
        public void tableChanged(TableModelEvent e) {
            if ((e.getType() == TableModelEvent.INSERT)
                            || (e.getType() == TableModelEvent.UPDATE)) {
                for (int x = e.getFirstRow(); x <= e.getLastRow(); x++) {
                    updateRowHeight(x);
                }
            }
        }
    }
}