package de.renew.ptchannel.nin;

import de.renew.net.Place;
import de.renew.net.TransitionInscription;
import de.renew.ptchannel.single.SinglePTNetWithChannelCompiler;
import de.renew.shadow.ShadowArc;
import de.renew.shadow.ShadowDeclarationNode;
import de.renew.shadow.ShadowInscription;
import de.renew.shadow.ShadowNet;
import de.renew.shadow.ShadowPlace;
import de.renew.shadow.SyntaxException;

import java.util.Collection;

/**
 * This is the compiler for P/T nets which can communicate
 * with other net instances.
 * The only thing it does is delegating all
 * compilations to its parent class, except for system nets,
 * which get compiled with a {@link de.renew.formalism.java.JavaNetCompiler}.
 *
 * @author Lukas Voß
 */
public class SinglePTNetInNetCompiler extends SinglePTNetWithChannelCompiler {

    /**
     * Creates a new <code>SinglePTNetInNetCompiler</code> instance.
     */
    public SinglePTNetInNetCompiler() {
        super();
    }

    @Override
    public String checkTransitionInscription(String inscription,
                                             boolean special,
                                             ShadowNet shadowNet)
            throws SyntaxException {
        if(shadowNet.getName().equals("SystemNet")) {
            return transitionCompiler.checkTransitionInscription(inscription, special, shadowNet);
        }
        else {
            return super.checkTransitionInscription(inscription, special, shadowNet);
        }
    }

    @Override
    public String checkPlaceInscription(String inscr, boolean special,
                                        ShadowNet shadowNet)
            throws SyntaxException {
        if(shadowNet.getName().equals("SystemNet")) {
            return transitionCompiler.checkPlaceInscription(inscr, special, shadowNet);
        }
        else {
            return super.checkPlaceInscription(inscr, special, shadowNet);
        }
    }

    @Override
    public String checkArcInscription(String inscr, boolean special,
                                      ShadowNet shadowNet)
            throws SyntaxException {
        if(shadowNet.getName().equals("SystemNet")) {
            return transitionCompiler.checkArcInscription(inscr, special, shadowNet);
        }
        else {
            return super.checkArcInscription(inscr, special, shadowNet);
        }
    }

    @Override
    protected void compilePlaceInscriptions(ShadowPlace shadowPlace,
                                                     Place place)
            throws SyntaxException {
        if(shadowPlace.getNet().getName().equals("SystemNet")) {
            transitionCompiler.compilePlaceInscriptions(shadowPlace, place);
        }
        else {
            super.compilePlaceInscriptions(shadowPlace, place);
        }
    }

    @Override
    protected void compileArc(ShadowArc shadowArc)
            throws SyntaxException {
        if(shadowArc.getNet().getName().equals("SystemNet")) {
            transitionCompiler.compileArc(shadowArc, lookup);
        }
        else {
            super.compileArc(shadowArc);
        }
    }

    @Override
    protected Collection<TransitionInscription> compileTransitionInscription(ShadowInscription inscription)
            throws SyntaxException {
        if(inscription.getNet().getName().equals("SystemNet")) {
            return transitionCompiler.compileTransitionInscription(inscription, lookup);
        }
        else {
            return super.compileTransitionInscription(inscription);
        }
    }

    public void parseDeclarations(ShadowNet shadowNet)
            throws SyntaxException {
        if(shadowNet.getName().equals("SystemNet")) {
            transitionCompiler.parseDeclarations(shadowNet);
        }
        else {
            super.parseDeclarations(shadowNet);
        }
    }

    protected ShadowDeclarationNode findDeclarationNode(ShadowNet shadowNet)
            throws SyntaxException {
        if(shadowNet.getName().equals("SystemNet")) {
            return transitionCompiler.findDeclarationNode(shadowNet);
        }
        else {
            return super.findDeclarationNode(shadowNet);
        }
    }
}
